// Copyright (C) 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

function opustc_couplexrun ( plexfile , outputfile , castembin , dgibifile , verbose)
  //   Performs the Couplex simulation
  //
  // Calling Sequence
  //   opustc_couplexrun ( plexfile , outputfile , castembin , dgibifile , verbose)
  //
  // Parameters
  // plexfile : a 1-by-1 matrix of strings, the design of experiments file
  // outputfile : a 1-by-1 matrix of strings, the output file
	// castembin : a 1-by-1 matrix of strings, the full path to the CASTEM executable
	// dgibifile : a 1-by-1 matrix of strings, the full path to the dgibi template data file.
	// verbose: a 1-by-1 matrix of doubles, integer value, the verbose mode, set verbose=0 to get no messages, set verbose=1 to get more messages.
  //
  // Description
  //   Launch Cast3m and perform the COUPLEX simulation.
  //
  //   Prerequirements: 
  //   The Cast3m software must be installed on the machine.
  //
  //   http://www-cast3m.cea.fr/cast3m/index.jsp
  //
  //   In order to check that cast3m is correctly installed on your
  //   system, open a terminal and type :
  //
  //   castem09.bat
  //
  //   This program is an interface to the "opuscouplex" program created
  //   by Jean-Marc Martinez to call CASTEM onto the COUPLEX benchmark.
  //
  // Examples
	// // CASTEM prins messages in the console.
	// consolebox("on")
	// // or F12.
	//
	// // Perform one simulation
  // X = [25.0 3.0e-6 6.0 3.0e-5 1e-6 5.0e-4 50. 50. 50. 50. 21. 51. 3. 110. 160. ];
  // [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( X )
  // verbose = 0;
  // cwd=pwd();
  // chdir(cplxdir);
  // opustc_couplexrun ( plexfile , outputfile , castembin , dgibifile , verbose )
  // chdir(cwd);
  //
  // // Get more messages
  // X = [25.0 3.0e-6 6.0 3.0e-5 1e-6 5.0e-4 50. 50. 50. 50. 21. 51. 3. 110. 160. ];
  // [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( X )
  // verbose = 1;
  // cwd=pwd();
  // chdir(cplxdir);
  // opustc_couplexrun ( plexfile , outputfile , castembin , dgibifile , verbose )
  // chdir(cwd);
  //
  // Authors
  //   Michael Baudin, 2011
  //
  // Bibliography
    //   "OPUS, Cas test CEA, Propagation des incertitudes dans le probleme COUPLEX-1", JM. Martinez, Ph. Montarnal, M. Lamoureux, O. Le Maitre, 6 avril 2010
    //   "Analyse d'incertitudes : application de la theorie des polynomes du chaos a l'analyse de surete du stockage de dechets nucleaires", Ecole Centrale Paris, Paul Balanca, Anne-Celine Boulanger, Marc Dupin, Cyril Labbe, Etude encadree par Florian De Vuyst
  //
  
endfunction

