// Copyright (C) 2009 - 2011 - Digiteo - Michael Baudin
// Copyright (C) 2009 - 2011 - CEA - Jean-Marc Martinez
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

extern "C" {
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"
#include "stdlib.h"
#include "gw_opustc.h"
#include "sciprint.h"
}


/* ==================================================================== */

#include "opustc_gwsupport.hxx" 
#include "opuscouplex.h" 

int sci_opuscouplex_startup_flag = 0;

void sci_opuscouplex_errorfunction ( char * message );
void sci_opuscouplex_messagefunction ( char * message );

/* ==================================================================== */
// 
// opustc_couplexrun ( plexfile , outputfile , castembin , dgibifile ) 
//   Performs the simulation.
// Arguments
  // plexfile : a 1-by-1 matrix of strings, the design of experiments file
  // outputfile : a 1-by-1 matrix of strings, the output file
	// castembin : a 1-by-1 matrix of strings, the full path to the CASTEM executable
	// dgibifile : a 1-by-1 matrix of strings, the full path to the dgibi template data file.
//
int sci_opustc_couplexrun (char *fname)
{
	char * plexfile = NULL;
	char * outputfile = NULL;
	char * castembin = NULL;
	char * dgibifile = NULL;
	char ** stringdata = NULL;
	int nRows, nCols;
	int result = 0;
	int ivar;
	int verbose;

		// Connect the Nisp message function to Scilab's print
	if (sci_opuscouplex_startup_flag==0) {
		opuscouplex_msgsetfunction ( sci_opuscouplex_messagefunction );
		opuscouplex_errorsetfunction ( sci_opuscouplex_errorfunction );
		sci_opuscouplex_startup_flag = 1;
	}

	CheckRhs(5,5) ;
	CheckLhs(0,1) ;

	// Get design of experiments file
	ivar = 1;
	opustc_AssertVarType(fname , ivar , sci_strings );
	GetRhsVar( ivar, MATRIX_OF_STRING_DATATYPE, &nRows,   &nCols,   &stringdata);
	opustc_AssertNRows ( fname , ivar , nRows , 1 );
	opustc_AssertNColumns ( fname , ivar , nCols , 1 );
	plexfile = stringdata[0];

	// Get outputfile file
	ivar = 2;
	opustc_AssertVarType(fname , ivar , sci_strings );
	GetRhsVar( ivar, MATRIX_OF_STRING_DATATYPE, &nRows,   &nCols,   &stringdata);
	opustc_AssertNRows ( fname , ivar , nRows , 1 );
	opustc_AssertNColumns ( fname , ivar , nCols , 1 );
	outputfile = stringdata[0];

	// Get castembin path
	ivar = 3;
	opustc_AssertVarType(fname , ivar , sci_strings );
	GetRhsVar( ivar, MATRIX_OF_STRING_DATATYPE, &nRows,   &nCols,   &stringdata);
	opustc_AssertNRows ( fname , ivar , nRows , 1 );
	opustc_AssertNColumns ( fname , ivar , nCols , 1 );
	castembin = stringdata[0];

	// Get dgibifile file
	ivar = 4;
	opustc_AssertVarType(fname , ivar , sci_strings );
	GetRhsVar( ivar, MATRIX_OF_STRING_DATATYPE, &nRows,   &nCols,   &stringdata);
	opustc_AssertNRows ( fname , ivar , nRows , 1 );
	opustc_AssertNColumns ( fname , ivar , nCols , 1 );
	dgibifile = stringdata[0];


	// Get verbose file
	opustc_GetOneInteger ( fname , 5 , &verbose )

	// Perform the simulation
	opuscouplex_verboselevelset ( verbose );
	opuscouplex_main ( plexfile , outputfile , castembin , dgibifile );

	opustc_CreateLhsInteger ( 1 , result );

	return 0;
}
/* ==================================================================== */

// sci_opuscouplex_errorfunction --
//   The error callback used by opuscouplex.
//   Redirect the message to Scilab's error function.
void sci_opuscouplex_errorfunction ( char * message ) {
	Scierror(999,_("opuscouplex: Error at the library level:\n%s\n") , message );
}

// sci_opuscouplex_messagefunction --
//   The error callback used by opuscouplex.
//   Redirect the message to Scilab's error function.
void sci_opuscouplex_messagefunction ( char * message ) {
	sciprint( message );
}

