// Copyright (C) 2009 - 2011 - Digiteo - Michael Baudin
// Copyright (C) 2009 - 2011 - CEA DM2S/SFME/LGLS - Jean-Marc Martinez
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <cstring>
#include <sstream>
#include <string>
#include <cstdlib>


using namespace std;

#include "opuscouplex.h"

//
// Internal variables
//

char BIN_CASTEM[1024];
char FICHIER_REFERENCE[1024];

void (* opuscouplex_messagefunction)(char * message) = NULL;
void (* opuscouplex_errorfunction)(char * message) = NULL;
int opuscouplex_verboselevel = 0;

int const NL = 43;      // nombre de lignes du fichier resu_conc.dat de CASTEM

int const NC = 8;       // nombre de points ou est analysee la concentration

int const NF = 6;	// nombre de flux

int const NT = 12;       // nombre d'instants ou l'analyse est realisee
double temps[] = { 1.e3, 5.e3, 1.e4, 2.e4, 5.e4, 1.e5, 2.e5, 5.e5, 1.e6, 2.e6, 5.e6, 1.e7 };

const int NPAR = 15; // Nombre de parametres
// Nom des parametres
const char *ParamName[]={
	(char *) "PerDog", (char *) "PerCla", (char *) "PerLim", (char *) "PerMar",
	(char *) "DifCla", (char *) "DifMar", (char *) "DiLoDo", (char *) "DiLoLi",
	(char *) "DiTrDo", (char *) "DiTrLi", (char *) "DeCH21", (char *) "DeCH31",
	(char *) "DeCH16", (char *) "DeCH25", (char *) "DeCH34"
};

/////////////////////////////////////////////////////////
//
// Functions used within this file
//
void opuscouplex_error ( char * message );
void opuscouplex_error ( const string & str );
void opuscouplex_verbosemessage ( char * message );
void opuscouplex_verbosemessage ( const string & str );
void opuscouplex_message ( char * message );
void opuscouplex_message ( const string & str );

/////////////////////////////////////////////////////////
//
// Implementations of the functions.
//
int opuscouplex_testName(char * name) {
	for(int i=0;i<15;i++) {
		if(!strcmp(ParamName[i],name)) {
			return(1);
		}
	}
	return(0);
}

void opuscouplex_pretraitement(char * FicParam, int * ierr ) {
	char buffer[1024];
	char name[NPAR+1][1024];
	double value[NPAR+1];
	ostringstream msg;
	*ierr=0;

	ifstream param(FicParam,ios::in);
	if(!param) { 
		msg << "Echec a l'ouverture de " << FicParam << endl; 
		opuscouplex_error ( msg.str() );
		*ierr=1;
		return;
	}
	int nv=0;
	while(param >> name[nv]) {
		param >> value[nv];
		nv++;
		if(nv>NPAR)  { 
			msg << "Erreur - le fichier " << FicParam << " comporte un nombre de parametres > " << NPAR << " !!" << endl; 
			opuscouplex_error ( msg.str() );
			*ierr=1;
			return;
		}
	}
	param.close();

	// test des noms des parametres
	for(int i=0;i<nv;i++) {
		if(!opuscouplex_testName(name[i])) {
			msg << "Fichier " << FicParam << " - Erreur : le parametre << " << name[i] << " >> est inconnu" << endl;
			opuscouplex_error ( msg.str() );
			*ierr=1;
			return;
		}
	}

	// Constitution du nom du fichier de reference
	strcpy(buffer,FICHIER_REFERENCE);
	ifstream JddCouplexRef(buffer,ios::in);
	if(!JddCouplexRef) { 
		msg << "Echec a l'ouverture de " << buffer << endl; 
		opuscouplex_error ( msg.str() );
		*ierr=1;
		return;
	}

	ofstream JddCouplex("couplex.dgibi",ios::out);
	if(!JddCouplex) { 
		msg << "Creation du fichier couplex.dgibi impossible\n"; 
		opuscouplex_error ( msg.str() );
		*ierr=1;
		return;
	}

	while(JddCouplexRef.getline(buffer,1024,'\n')) {
		if(!strcmp("*FlagInsertion",buffer)) {
			JddCouplex << endl;
			JddCouplex << "*==========================================================="<<endl;
			JddCouplex << "*Debut Insertion nouvelles valeurs des parametres incertains"<< endl;
			for(int i=0;i<nv;i++) {
				JddCouplex << name[i] << " = " << value[i] << " ;" << endl;
			}
			JddCouplex << "*Fin Insertion"<< endl;
			JddCouplex << "*============="<< endl;
			JddCouplex << endl;
		}
		else {
			JddCouplex << buffer << endl;
		}
	}
	JddCouplexRef.close();
	JddCouplex.close();
}

double opuscouplex_interpolation(double *xt, double *yt, int n, double t) {
	int i;
	if(t <= xt[0])   {
		return yt[0];
	}
	if(t >= xt[n-1]) {
		return yt[n-1];
	}
	for(i=1;i<n;i++) {
		if(t<xt[i]) {
			break;
		}
	}
	i=i-1;
	return(yt[i]+(t-xt[i])*(yt[i+1]-yt[i])/(xt[i+1]-xt[i]));
}

void opuscouplex_postraitement(int * ierr) {
	char buffer[1024];
	int i,j;
	double yc[NL][NC];
	double yf[NL][NF];
	double zc[NT][NC];
	double zf[NT][NF];
	double xt[NL];
	double yt[NL];
	ostringstream msg;

	* ierr = 0;

	ifstream resu_concentration("resu_conc.dat",ios::in);
	if(!resu_concentration) { 
		msg << "Echec a l'ouverture de resu_conc.dat" << endl; 
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	} 
	resu_concentration.getline(buffer,1024,'\n'); // on saute la premiere ligne
	for(i=0;i<NL;i++) {
		resu_concentration >> xt[i]; // temps CASTEM
		for(j=0;j<NC;j++) {
			resu_concentration >> yc[i][j];
		}
	}
	resu_concentration.close();

	ifstream resu_flux("resu_flux.dat",ios::in);
	if(!resu_flux) { 
		msg << "Echec a l'ouverture de resu_flux.dat" << endl; 
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	} 
	resu_flux.getline(buffer,1024,'\n'); // on saute la premiere ligne
	for(i=0;i<NL;i++) {
		resu_flux >> xt[i]; // temps CASTEM
		for(j=0;j<NF;j++) {
			resu_flux >> yf[i][j];
		}
	}
	resu_flux.close();

	// PostTraitement par interpolation
	for(j=0;j<NC;j++) {
		for(i=0;i<NL;i++) {
			yt[i]=yc[i][j];
		}
		for(i=0;i<NT;i++) {
			zc[i][j]=opuscouplex_interpolation(xt,yt,NL,temps[i]);
		}
	}

	for(j=0;j<NF;j++) {
		for(i=0;i<NL;i++) {
			yt[i]=yf[i][j];
		}
		for(i=0;i<NT;i++) {
			zf[i][j]=opuscouplex_interpolation(xt,yt,NL,temps[i]);
		}
	}

	ofstream sortie("resu_post.dat",ios::out);
	if(!sortie) { 
		msg << "Echec a l'ouverture de resu_post.dat " << endl; 
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	}
	sortie.setf(ios::scientific, ios::floatfield);
	sortie.precision(6);

	sortie.width(16); 
	sortie << "#TEMPS(annees)";
	sortie.width(16); 
	sortie << "CONC/P1";
	sortie.width(16); 
	sortie << "CONC/P2";
	sortie.width(16); 
	sortie << "CONC/P3";
	sortie.width(16); 
	sortie << "CONC/P4";
	sortie.width(16); 
	sortie << "CONC/P5";
	sortie.width(16); 
	sortie << "CONC/P6";
	sortie.width(16); 
	sortie << "CONC/P7";
	sortie.width(16); 
	sortie << "CONC/P8";
	sortie.width(16); 
	sortie << "FLUX/G_DOG";
	sortie.width(16); 
	sortie << "FLUX/G_LIM";
	sortie.width(16); 
	sortie << "FLUX/H_CLAY";
	sortie.width(16); 
	sortie << "FLUX/B_CLAY";
	sortie.width(16); 
	sortie << "FLUX/CONTOUR";
	sortie.width(16); 
	sortie << "FLUX/H_MARL";
	sortie << endl;
	for(i=0;i<NT;i++) {
		sortie.width(16); 
		sortie << temps[i];
		for(j=0;j<NC;j++) {
			sortie.width(16); 
			sortie << zc[i][j];
		}
		for(j=0;j<NF;j++) {
			sortie.width(16); 
			sortie << zf[i][j];
		}
		sortie << endl;
	}
	sortie.close();
}

void opuscouplex_mono(char * ficParam, int * ierr) {
	char cmd[1024], ligne[1024], fioderef[1024];
	ostringstream msg;

	*ierr = 0;

	//MB, 15/04/2009 : setenv("CASTEM",ENVIRONNEMENT_CASTEM,1);
	strcpy(cmd,BIN_CASTEM);
	//strcat(cmd," couplex.dgibi >> /dev/null \n");
	strcat(cmd," couplex.dgibi\n");

	// Recopie du fichier iode.dat (si besoin)
	if (!ifstream ("iode.dat")) {
		ofstream iode("iode.dat",ios::out);
		strcpy(fioderef,FICHIER_REFERENCE);
		strcat(fioderef,"/iode.dat");
		ifstream ioderef(fioderef,ios::in);
		if(!ioderef) {
			msg << "Erreur de lecture du fichier " << fioderef << endl; 
			opuscouplex_error ( msg.str() );
			*ierr = 1;
			return;
		}
		while(ioderef.getline(ligne,1024,'\n')) { 
			iode << ligne << endl; 
		}
		ioderef.close();
		iode.close();
	}

	opuscouplex_pretraitement(ficParam,ierr);
	if ( *ierr )  
	{
		return;
	}
	msg << "OPUS - Cas Test CEA - Bench COUPLEX : " << cmd << endl;
	opuscouplex_verbosemessage ( msg.str() );
	system(cmd);
	opuscouplex_postraitement(ierr);
}

void ReadOutput(double *z,int * ierr) {
	double time;
	ostringstream msg;
	*ierr = 0;

	ifstream resu_post("resu_post.dat",ios::in);
	if(!resu_post) {
		msg << "Echec a l'ouverture de resu_post.dat" << endl; 
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	}
	int i,j,k;

	// on saute la premiere ligne
	char str[1024];
	resu_post >> str; // temps
	for(j=0;j<NC;j++) {
		resu_post >> str; // concentration
	}
	for(j=0;j<NF;j++) {
		resu_post >> str; // flux
	}

	k=0;
	for(i=0;i<NT;i++) {
		resu_post >> time;
		for(j=0;j<NC;j++) {
			resu_post >> z[k++]; // concentration
		}
		for(j=0;j<NF;j++) {
			resu_post >> z[k++]; // flux
		}
	}
	resu_post.close();
}

void opuscouplex_multi(char *plan, char *reponse, int format, int * ierr) {
	int i,k;
	char str[1024];
	ostringstream msg;
	*ierr = 0;

	ifstream fplan(plan,ios::in);
	if(!fplan) { 
		msg << "Echec a l'ouverture de " << plan << endl; 
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	}
	int np, nx;
	fplan >> str >> nx;
	if(nx>NPAR) {
		msg << "Nombre de parametres " << nx << " > " << NPAR << endl;
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	}
	char ** name = new char * [nx];
	for(int i = 0; i < nx; i++) {
		name[i] = new char[1024];
	}

	for(i=0;i<nx;i++) {
		fplan >> name[i];
		if(!opuscouplex_testName(name[i])) {
			msg << "Fichier " << plan << " - Erreur : le parametre << " << name[i] << " >> est inconnu" << endl;
			opuscouplex_error ( msg.str() );
			*ierr = 1;
			return;
		}
	}
	double * x = new double[nx];
	fplan >> str >> np;

	/* Sauvegarde */
	double z[NT*(NC+NF)];
	int nr=NT*(NC+NF);
	//ofstream freponse(reponse,ios::app);
	ofstream freponse(reponse,ios::out);
	if(!freponse) { 
		msg << "Echec a l'ouverture de " << reponse << endl; 
		opuscouplex_error ( msg.str() );
		*ierr = 1;
		return;
	}
	freponse.setf(ios::scientific, ios::floatfield);
	freponse.precision(6);
	if(format == 0) {
		freponse << "#NombreSimulations " << np << endl;
		freponse << "#NombreSorties     " << nr << endl;
	}
	else {
		freponse << "#NombreSimulations " << np << endl;
		freponse << "#NombreEntrees     " << nx << endl;
		freponse << "#NombreSorties     " << nr << endl;
	}
	for(k=0;k<np;k++) {
		for(i=0;i<nx;i++) {
			fplan >> x[i];
			if(!fplan) { 
				freponse.close(); 
				msg << "Erreur de lecture dans le fichier " << plan << endl; 
				opuscouplex_error ( msg.str() );
				*ierr = 1;
				return;
			}
		}
		// Genere le fichier couplex.param
		ofstream param("couplex.param",ios::out);
		if(!param) { 
			msg << "Echec a l'ouverture de couplex.param" << endl; 
			opuscouplex_error ( msg.str() );
			*ierr = 1;
			return;
		}
		for(i=0;i<nx;i++) {
			param << name[i] << " " << x[i] << endl;
		}
		param.close();
		msg << "Calcul " << k + 1 << "(" << np << ") : ";
		opuscouplex_verbosemessage ( msg.str() );
		opuscouplex_mono((char *) "couplex.param",ierr);
		if ( *ierr )
		{
			return;
		}
		ReadOutput(z,ierr);
		if ( *ierr )
		{
			return;
		}
		// Sauvegarde
		if(format) {
			for(i=0;i<nx;i++) {
				freponse.width(16); 
				freponse << x[i];
			}
		}
		for(i=0;i<nr;i++) {
			freponse.width(16); freponse << z[i];
		}
		freponse << endl;
	}
	fplan.close();
	freponse.close();
}


void helpcouplex() {
	ostringstream msg;

	msg << endl;
	msg << "========================================"<<endl;
	msg << "Begin - Help Use Case OPUS CEA - COUPLEX"<< endl;
	msg << "========================================"<< endl;
	msg << endl;
	msg << "**** Cas 1 : Realisation d'une seule simulation :" << endl;
	msg <<endl;
	msg << "opuscea fileParam" << endl;
	msg << endl;
	msg << "**** Le fichier fileParam specifie les valeurs des parametres incertains disposes" << endl;
	msg << "**** par les sequences << nom_parametre valeur_parametre >>" <<endl;
	msg << "**** Exemple d'un fichier fileParam pour les 4 permeabilites"<<endl;
	msg << endl;
	msg << "PerDog 5.32826" << endl;
	msg << "PerCla 3e-06"<< endl;
	msg << "PerLim 6" << endl;
	msg << "PerMar 3.16228e-05" << endl;
	msg << endl;
	msg << "**** Cas 2 : Realisation d'un plan d'experiences numeriques" << endl;
	msg << endl;
	msg << "opuscea filePlanExperience fileReponse" << endl;
	msg << endl;
	msg << "**** filePlanExperience : fichier contenant le plan d'experiences"<< endl;
	msg << "**** Exemple d'un fichier filePlanExperience d'un plan d'experiences sur les 4permeabilites : " << endl;
	msg << "#Parametres 4"<<endl;
	msg << "PerDog         PerCla         PerLim         PerMar"<<endl;
	msg << "#Simulations 193"<<endl;
	msg << "2.500000e+01   3.000000e-06   6.000000e+00   3.162278e-05" << endl;
	msg << "2.500000e+01   3.000000e-06   6.000000e+00   7.714332e-05" << endl;
	msg << "2.500000e+01   3.000000e-06   6.000000e+00   1.296288e-05" << endl;
	msg << "........................................................." << endl;
	msg << "**** fileReponse : fichier issu des realisation du plan d'experiences"<< endl;
	msg << "**** Ecriture sous forme matricielle : ligne => calcul, colonne => valeurs des variables" << endl;
	msg << "**** Exemple du fichier reponse : "<< endl;
	msg << "#NombreSimulations 81"<<endl;
	msg << "#NombreSorties     168"<<endl;
	msg << "**** puis suivent 81 lignes les valeurs des 168 grandeurs d'interet"<<endl;
	msg << "**** les 8 concentrations puis les 6 flux en t=1e3"<<endl;
	msg << "**** puis a la ligne suivante les 8 concentrations puis les 6 flux en t=5e3"<<endl;
	msg << "**** ... etc ... jusqu'a la derniere ligne : "<<endl;
	msg << "**** correspondant aux 8 concentrations et aux 6 flux en t=1e7"<<endl;
	msg << "**** ORDRE des concentrations : "<<endl;
	msg << "****     P1(x=19000m,z=346m), P2(x=10900m,z=495m), P3(x=30m,z=495m),..."<<endl;
	msg << "**** ... P4(x=19000m,z=195m), P5(x=10900m,z=105m), P6(x=30, z=105m),..."<<endl;
	msg << "**** ... P7(x=50m,   z=692m), P8(x=24800m,z=692m)"<<endl;
	msg << "**** ORDRE des flux : "<<endl;
	msg << "****     Gauche DOGGER, Gauche LIME, Haut CLAY, Bas CLAY, Contour, Haut MARL"<<endl;
	msg << endl;
	msg << "**** On peut recopier les entrees du fichier filePlanexperience dans le fichier"<<endl;
	msg << "**** fileReponse par la commande "<<endl;
	msg << endl;
	msg << "opuscea filePlanExperience fileReponse 1"<<endl;
	msg << endl;
	msg << "**** Exemple d'un fichier fileReponse dans ce cas :"<< endl;
	msg << "#NombreSimulations 81"<<endl;
	msg << "#NombreEntrees     4"<<endl;
	msg << "#NombreSorties     168"<<endl;
	msg << "**** puis suivent 81 lignes. Pour chaque ligne, les valeurs des entrees "<<endl;
	msg << "**** (ici au nombre de 4) precedent les valeurs des 168 variables d'interet"<<endl;
	msg << "======================================"<<endl;
	msg << "End - Help Use Case OPUS CEA - COUPLEX"<<endl;
	msg << "======================================"<<endl;

	opuscouplex_message ( msg.str() );
}


void opuscouplex_main ( char * plan , char * reponse , char * castembin , char * dgibifile ) {
	int ierr;
	int format;

	strcpy(BIN_CASTEM,castembin);
	strcpy(FICHIER_REFERENCE,dgibifile);
	format = 0;
	opuscouplex_multi(plan,reponse,format, &ierr);
}

void opuscouplex_message ( char * message ) {
	if (opuscouplex_messagefunction==NULL) {
		cout << message;
	} 
	else 
	{
		opuscouplex_messagefunction(message);
	}
}

void opuscouplex_message ( const string & str ) {
	char * cstr;
	cstr = new char [str.size()+1];
	strcpy (cstr, str.c_str());
	opuscouplex_message ( cstr );
	delete [] cstr;
}

void opuscouplex_verbosemessage ( char * message ) {
	if (opuscouplex_verboselevel==1)
	{
		opuscouplex_message ( message );
	}
}

void opuscouplex_verbosemessage ( const string& str ) {
	char * cstr;
	cstr = new char [str.size()+1];
	strcpy (cstr, str.c_str());
	opuscouplex_verbosemessage ( cstr );
	delete [] cstr;
}


// TODO : generate exceptions instead
// TODO : manage multi-thread issues by global variable opuscouplex_errorfunction
void opuscouplex_error ( char * message ) {
	if (opuscouplex_errorfunction==NULL) {
		opuscouplex_message ( "opuscouplex: Error !" );
		opuscouplex_message ( message );
		exit(1);
	} 
	else 
	{
		opuscouplex_errorfunction ( message );
	}
}

void opuscouplex_error ( const string & str ) {
	char * cstr;
	cstr = new char [str.size()+1];
	strcpy (cstr, str.c_str());
	opuscouplex_error ( cstr );
	delete [] cstr;
}

void opuscouplex_msgsetfunction ( void (* f)(char * message) ) {
	opuscouplex_messagefunction = f;
}

void opuscouplex_errorsetfunction ( void (* f)(char * message) ) {
	opuscouplex_errorfunction = f;
}

void opuscouplex_verboselevelset ( int newlevel) {
	if (newlevel == 0) {
		opuscouplex_verboselevel = 0;
	}
	else if (newlevel == 1)
	{
		opuscouplex_verboselevel = 1;
	}
	else
	{
		ostringstream msg;
		msg << "opuscouplex - ERROR" << endl;
		msg << "Unknown verbose level " << newlevel << endl;
		opuscouplex_error ( msg.str() );
	}
}

int opuscouplex_verboselevelget () {
	return opuscouplex_verboselevel;
}
